;; Copyright (C) 2023  Felisp
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU Affero General Public License as published by
;; the Free Software Foundation, version 3 of the License.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU Affero General Public License for more details.
;;
;; You should have received a copy of the GNU Affero General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>.

(ns rss-thread-watch.core
  (:require [clojure.java.io :As io]
            [ring.adapter.jetty :as jetty]
            [ring.middleware.params :as rp]
            [rss-thread-watch.watcher :as watcher]
            [rss-thread-watch.feed-generator :as feed])
  (:gen-class))

;; Internal default config
(def CONFIG-DEFAULT
  "Internal default config"
  {:port 6969
   :default-board "/mlp/"
   :enable-board-listing true
   :board-disabled-message "This board is not enabled for feed generation.\n\nYou can contact me here: [contact]"
   :target "https://api.4chan.org"
   :boards-defaults {:refresh-rate 300
                     :starting-page 7
                     :default-chod 94}
   :boards-enabled {"/mlp/" {}
                    "/g/" {}}})

(defn set-interval
  "Calls function every ms"
  [callback ms]
  (future (while true (do (try
                            (callback)
                            (println "Recached")
                            (catch Exception e
                              (binding [*out* *err*]
                                (println "Error while updating cache: " e ", retrying in " (/ ms 1000 60) " minutes"))))
                          (Thread/sleep ms)))))

(defn -main
  "Entry point, starts webserver"
  [& args]
  ;; Parse and validate config
  ;; Think of a way to start repeated download for every catalog efficiently
  (set-interval (fn []
                  (println "Starting cache update")
                  (watcher/update-thread-cache! (:target CONFIG) (:starting-page CONFIG)))
                (* 1000 (:refresh-delay CONFIG)))
  (jetty/run-jetty (rp/wrap-params feed/http-handler) {:port (:port CONFIG)
                                                       :join? true}))

;; Docs: https://github.com/ring-clojure/ring/wiki/Getting-Started
(defn repl-main
  "Development entry point"
  []
  (jetty/run-jetty (rp/wrap-params #'feed/http-handler)
                   {:port (:port CONFIG)
                    ;; Dont block REPL thread
                    :join? false}))
;; (repl-main)
;; Single cache update for repl
;; (watcher/update-thread-cache! (:target CONFIG) (:starting-page CONFIG))
